#-----------------------------------------------------------------------------
# Player
# Copyright (c) 2009 David G. Drumright (Ockham)
# Aug 14, 2009
#-----------------------------------------------------------------------------
# Reads the DAT created by Recorder.
#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import poser
import pickle
import os
scene = poser.Scene()
Fig=scene.CurrentFigure()

#-----------------------------------------------------------------------------

global MainRecord
MovableParameters=[poser.kParmCodeXTRAN, poser.kParmCodeYTRAN,
poser.kParmCodeZTRAN, poser.kParmCodeXROT, poser.kParmCodeYROT,
poser.kParmCodeZROT]

#-----------------------------------------------------------------------------

class App:
	def __init__(self, master, textMessage):
		self.master = master
		master.title("Player")

		self.TopFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.TopFrame.grid(row=1,column=0)
		self.BotFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.BotFrame.grid(row=3,column=0)

		self.StatusEntry = Entry(self.TopFrame,width=70)
		self.StatusEntry.grid(row=0,column=0,columnspan=3)
		self.StatusEntry.insert(0,"First pick a DAT file made by Recorder.")

		# Go/nogo buttons.
		self.ButtonName = Button(self.TopFrame, text="Pick file", command=self.HandleName)
		self.ButtonName.grid(row=1, column=0,sticky=E+W)

		Label(self.TopFrame, text="Start frame:" ).grid(row=2, sticky=W)
		self.StartEntry = Entry(self.TopFrame, width=5)
		self.StartEntry.grid(row=2, column=1)
		self.StartEntry.insert(0,str(1))

		Label(self.TopFrame, text="End frame:" ).grid(row=3, sticky=W)
		self.EndEntry = Entry(self.TopFrame, width=5)
		self.EndEntry.grid(row=3, column=1)
		self.EndEntry.insert(0,str(scene.NumFrames()))

		self.ButtonGo = Button(self.BotFrame, text="Go absolute", command=self.HandleGoAbs)
		self.ButtonGo.grid(row=0, column=0)
		self.ButtonGo = Button(self.BotFrame, text="Go additive", command=self.HandleGoAdd)
		self.ButtonGo.grid(row=0, column=1)
		self.ButtonCancel = Button(self.BotFrame, text="Cancel", command=self.HandleCancel)
		self.ButtonCancel.grid(row=0, column=2)

		self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

	# - - - - - - - - - - - - - - - - - -

	def ShowStatus(self,S): # Just saving repetition
		self.StatusEntry.delete(0,END)
		self.StatusEntry.insert(0,S)
		self.StatusEntry.update_idletasks()

	# - - - - - - - - - - - - - - - - - -
	def HandleName(self):
		global FullFileName
		global MainRecord


		# Pop up file box to get name.
		BasePathUncut=poser.AppLocation()
		BasePath=BasePathUncut[:(-len('poser.exe'))]
		MyPath=os.path.normpath(BasePath+'/runtime/*.DAT') # Use this for release
		fn=tkFileDialog.askopenfilename(initialfile=MyPath, filetypes=[("DAT file", "*.DAT *.dat")] )

		if not fn:
			self.ShowStatus("No file selected. Try again.")
			return 0

		# Open the file
		FullFileName=os.path.abspath(fn)
		fp=open(FullFileName,'r')
		MainRecord=pickle.load(fp)
		fp.close()
		self.ShowStatus("Now set the start and end frame then hit Go.")

	# - - - - - - - - - - - - - - - - - -

	def HandleGoAbs(self):
		global FullFileName
		global MainRecord
		# Convert the time.  The DAT file is in ms, so we need a factor
		# to convert ms to frames.	We have the desired total frames.
		# The total ms is the time part of the last thing in the array.
		SF=self.StartEntry.get()
		if SF: StartFrame=int(SF)
		else:  StartFrame=1
		EF=self.EndEntry.get()
		if EF: EndFrame=int(EF)
		else:  EndFrame=scene.NumFrames()
		TotalFrames=EndFrame-StartFrame+1


		# We need to do MS * Factor = Frame, so Factor must be Frame/MS.
		(TotalMS,dummy)=MainRecord[-1]	#Fetch the last member
		factorT=TotalFrames/float(TotalMS)
		#ActorsUsed=[] # Make a list of actors involved so we can avoid
		# touching the things that weren't moved. (Later)
		for (MS,TempDict) in MainRecord:
			# TempDict is a dict with key=actor name, value=(parm values)
			# where (parm values) is a list in standard order of the parm values
			# in the MovableParameters set.
			for N in TempDict.keys():
				ParmList=TempDict[N] # Same order as MovableParameters
				A=Fig.Actor(N)
				for i in range(len(MovableParameters)):
					try:
						Parm=A.ParameterByCode(MovableParameters[i])
						Parm.SetValue(ParmList[i])
					except:
						continue

				Frame=StartFrame+int(factorT * float(MS))
				scene.SetFrame(Frame-1)

		self.master.destroy()


	# - - - - - - - - - - - - - - - - - -
	def HandleGoAdd(self):
		global FullFileName
		global MainRecord
		# Convert the time.  The DAT file is in ms, so we need a factor
		# to convert ms to frames.	We have the desired total frames.
		# The total ms is the time part of the last thing in the array.
		SF=self.StartEntry.get()
		if SF: StartFrame=int(SF)
		else:  StartFrame=1
		EF=self.EndEntry.get()
		if EF: EndFrame=int(EF)
		else:  EndFrame=scene.NumFrames()
		TotalFrames=EndFrame-StartFrame+1


		# We need to do MS * Factor = Frame, so Factor must be Frame/MS.
		(TotalMS,X,Y,Status)=MainRecord[-1]  #Fetch the last member
		factorT=TotalFrames/float(TotalMS)
		#ActorsUsed=[] # Make a list of actors involved so we can avoid
		# touching the things that weren't moved. (Later)
		for (MS,TempDict) in MainRecord:
			# TempDict is a dict with key=actor name, value=(parm values)
			# where (parm values) is a list in standard order of the parm values
			# in the MovableParameters set.
			for N in TempDict.keys():
				ParmList=TempDict[N] # Same order as MovableParameters
				A=Fig.Actor(N)
				for i in range(len(MovableParameters)):
					try:
						Parm=A.ParameterByCode(MovableParameters[i])
						Parm.SetValue(Parm.Value()+ParmList[i]) # only diff from abs
					except:
						continue

				Frame=StartFrame+int(factorT * float(MS))
				scene.SetFrame(Frame-1)

		self.master.destroy()
	# - - - - - - - - - - - - - - - - - -

	def HandleCancel(self):
		self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
