#-----------------------------------------------------------------------------
# Recorder
# Copyright (c) 2009 David G. Drumright (Ockham)
# Aug 14, 2009
#-----------------------------------------------------------------------------

import poser
from Tkinter import *
import tkFileDialog
import pickle
import os

#-----------------------------------------------------------------------------

global FullFileName
global StartTime
global Ticking

FullFileName=''
StartTime=0
ElapsedTime=0
Ticking=0
scene=poser.Scene()
Fig=scene.CurrentFigure()

# To save time, use only the six mouse-movable parameters.

MovableParameters=[poser.kParmCodeXTRAN, poser.kParmCodeYTRAN,
poser.kParmCodeZTRAN, poser.kParmCodeXROT, poser.kParmCodeYROT,
poser.kParmCodeZROT]

MainRecord=[]  # start the main array
# Each element of this array is: (Time,[parms changed])
# where Time is in milliseconds from the starting click.
# and [parms changed] is a list of (actor,parm,value) tuples

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
	def __init__(self, master, textMessage):
		self.master = master
		master.title("Record")

		self.TopFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.TopFrame.grid(row=1,column=0)
		self.BotFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.BotFrame.grid(row=3,column=0)

		self.StatusEntry = Entry(self.TopFrame,width=70)
		self.StatusEntry.grid(row=0,column=0,columnspan=2)
		self.StatusEntry.insert(0,"Select one figure. Hit start; move things; hit end.")
		self.Clock=Entry(self.TopFrame,width=9)
		self.Clock.grid(row=0,column=2)
		self.Clock.insert(0,str(0.0))


		# Go/nogo buttons.
		self.ButtonStart = Button(self.BotFrame, text="Start", command=self.HandleStart)
		self.ButtonStart.grid(row=0, column=0,padx=2,pady=2)
		self.ButtonEnd = Button(self.BotFrame, text="End", command=self.HandleEnd)
		self.ButtonEnd.grid(row=0, column=1,padx=2,pady=2)

		self.ButtonName = Button(self.BotFrame, text="Name file", command=self.HandleName)
		self.ButtonName.grid(row=0, column=2,padx=2,pady=2)

		self.ButtonCancel = Button(self.BotFrame, text="Cancel", command=self.HandleCancel)
		self.ButtonCancel.grid(row=0, column=3,padx=2,pady=2)

		self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

	# - - - - - - - - - - - - - - - - - -

	def ShowStatus(self,S): # Just saving repetition
		self.StatusEntry.delete(0,END)
		self.StatusEntry.insert(0,S)
		self.StatusEntry.update_idletasks()

	# - - - - - - - - - - - - - - - - - -

	def HandleStart(self):
		global StartTime,ElapsedTime,MainRecord,Ticking
		StartTime=0  # Set the start time
		ElapsedTime=0  # Set the elapsed time
		Ticking=1

	# - - - - - - - - - - - - - - - - - -

	def HandleTick(self):
		# Gets each tick
		global MainRecord

		if not Ticking: return

		for A in Fig.Actors():
			TempDict={} # is a dict with key=actor name, value=(parm values)
			# where (parm values) is a list in standard order of the parm values
			# in the MovableParameters set.
			N=A.Name()
			ParmList=[]
			for P in MovableParameters:
				try:
				  Parm=A.ParameterByCode(P)
				  ParmList.append(Parm.Value())
				except:
				  continue
			TempDict[N]=ParmList
			MainRecord.append((ElapsedTime,TempDict))

	# - - - - - - - - - - - - - - - - - -

	def HandleEnd(self):
		global MainRecord,Ticking
		# Rpress does only one thing: ends the action.
		Ticking=0

	# - - - - - - - - - - - - - - - - - -

	def HandleName(self):
		global FullFileName
		# Pop up file box to get name.
		BasePathUncut=poser.AppLocation()
		BasePath=BasePathUncut[:(-len('poser.exe'))]
		MyPath=os.path.normpath(BasePath+'/runtime/*.DAT') # Use this for release
		#fn=tkFileDialog.asksaveasfilename(initialfile=MyPath, filetypes=[("DAT file", "*.DAT")] )
		fn=tkFileDialog.asksaveasfilename(initialfile=MyPath, defaultextension=[("DAT file", "*.DAT *.dat")] )

		if not fn:
			self.ShowStatus("No file selected. Try again.")
			return 0

		# Open the file
		FullFileName=os.path.abspath(fn)
		fp=open(FullFileName,'w')
		pickle.dump(MainRecord,fp)
		fp.close()
		self.master.destroy()


	# - - - - - - - - - - - - - - - - - -

	def Update(self):
		global ElapsedTime
		if Ticking:
			self.Clock.delete(0,END)
			self.Clock.insert(0,"%6.1f" % (float(ElapsedTime)/1000.0) )
			self.Clock.update_idletasks()
		root.lift()
		ElapsedTime=ElapsedTime+100
		self.HandleTick()

		root.after(100, self.Update) # This serves to move the clock

	# - - - - - - - - - - - - - - - - - -

	def HandleCancel(self):
		self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
