#-----------------------------------------------------------------------------
# Flanger
# Copyright (c) 2009 David G. Drumright (Ockham)
# Aug 14, 2009
# Pick up what happens when you move the timeline back and forth.
# The script won't yield to the real timeline, so we put a substitute
# timeline on the script itself.
#-----------------------------------------------------------------------------

import poser
from Tkinter import *
import tkFileDialog
import pickle
import os

#-----------------------------------------------------------------------------

global FullFileName
global StartTime
global Ticking
global MainRecord

FullFileName=''
StartTime=0
ElapsedTime=0
Ticking=0
scene=poser.Scene()


MainRecord=[]  # start the main array
# Each element of this array is: (Time,[parms changed])
# where Time is in milliseconds from the starting click.
# and [parms changed] is a list of (actor,parm,value) tuples

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
	def __init__(self, master, textMessage):
		self.master = master
		master.title("Flanger")
		self.TopFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.TopFrame.grid(row=1,column=0)
		self.TimeLine=Scale(self.master,from_=0,to=scene.NumFrames(),orient=HORIZONTAL,resolution=1,showvalue=0)
		self.TimeLine.grid(row=2, column=0,sticky=E+W)
		# Set the timeline at the approx position of the real timeline
		self.TimeLine.set(scene.Frame())
		self.BotFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
		self.BotFrame.grid(row=3,column=0)

		self.StatusEntry = Entry(self.TopFrame,width=80)
		self.StatusEntry.grid(row=0,column=0)
		self.StatusEntry.insert(0,"Hit start; slide the internal timeline; hit end.")



		# Go/nogo buttons.
		self.ButtonStart = Button(self.BotFrame, text="Start", command=self.HandleStart)
		self.ButtonStart.grid(row=0, column=0,padx=2,pady=2)
		self.ButtonEnd = Button(self.BotFrame, text="End", command=self.HandleEnd)
		self.ButtonEnd.grid(row=0, column=1,padx=2,pady=2)
		self.ButtonName = Button(self.BotFrame, text="Name file", command=self.HandleName)
		self.ButtonName.grid(row=0, column=2,padx=2,pady=2)

		self.ButtonCancel = Button(self.BotFrame, text="Cancel", command=self.HandleCancel)
		self.ButtonCancel.grid(row=0, column=3,padx=2,pady=2)

		self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

	# - - - - - - - - - - - - - - - - - -

	def ShowStatus(self,S): # Just saving repetition
		self.StatusEntry.delete(0,END)
		self.StatusEntry.insert(0,S)
		self.StatusEntry.update_idletasks()

	# - - - - - - - - - - - - - - - - - -

	def HandleStart(self):
		global StartTime,ElapsedTime,MainRecord,Ticking
		StartTime=0  # Set the start time
		ElapsedTime=0  # Set the elapsed time
		Ticking=1

	# - - - - - - - - - - - - - - - - - -

	def HandleTick(self):
		# Gets each tick
		global MainRecord
		if not Ticking: return

		# Pick up current setting of internal timeline and set the scene.
		f=self.TimeLine.get()
		scene.SetFrame(f)
		scene.DrawAll()
		# Record everything in the scene at this moment
		# TempList is a list of tuples.  Each tuple is (FigName,ActorName,ParmList)
		# where FigName is the name of the actor's figure if any, or None if
		# the actor isn't part of a figure; ActorName is the actor's name,
		# and ParmList is a list of ((ParmName,Value),(ParmName,Value),...)
		# for (FigName,ActorName,ParmList) in TempList:
		TempList=[]
		for A in scene.Actors():
			try:
				FigName=A.ItsFigure().Name()
			except:
				FigName=None
			ActorName=A.Name()
			ParmList=[]
			for P in A.Parameters():
				if P.Hidden(): continue
				try:
				  ParmList.append( (P.Name(),P.Value()) )
				except:
				  continue
			TempList.append((FigName,ActorName,ParmList))
		# One entry in MainRecord for each tick
		MainRecord.append((ElapsedTime,TempList))

	# - - - - - - - - - - - - - - - - - -

	def HandleEnd(self):
		global MainRecord,Ticking
		Ticking=0

	# - - - - - - - - - - - - - - - - - -

	def HandleName(self):
		global FullFileName
		# Pop up file box to get name.
		BasePathUncut=poser.AppLocation()
		BasePath=BasePathUncut[:(-len('poser.exe'))]
		MyPath=os.path.normpath(BasePath+'/runtime/*.DAT') # Use this for release
		#fn=tkFileDialog.asksaveasfilename(initialfile=MyPath, filetypes=[("DAT file", "*.DAT")] )
		fn=tkFileDialog.asksaveasfilename(initialfile=MyPath, defaultextension=[("DAT file", "*.DAT *.dat")] )

		if not fn:
			self.ShowStatus("No file selected. Try again.")
			return 0

		# Open the file
		FullFileName=os.path.abspath(fn)
		fp=open(FullFileName,'w')
		pickle.dump(MainRecord,fp)
		fp.close()
		self.master.destroy()


	# - - - - - - - - - - - - - - - - - -

	def Update(self):
		global ElapsedTime
		root.lift()
		ElapsedTime=ElapsedTime+100
		self.HandleTick()

		root.after(100, self.Update) # Tick at 10 FPS

	# - - - - - - - - - - - - - - - - - -

	def HandleCancel(self):
		self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
